// Includes
#include <avr/io.h>
#include <util/delay.h>
#include <inttypes.h>

/* 
  UART-Init: 
Berechnung des Wertes fr das Baudratenregister 
aus Taktrate und gewnschter Baudrate
*/
 
#ifndef F_CPU
/* In neueren Version der WinAVR/Mfile Makefile-Vorlage kann
   F_CPU im Makefile definiert werden, eine nochmalige Definition
   hier wuerde zu einer Compilerwarnung fuehren. Daher "Schutz" durch
   #ifndef/#endif 
 
   Dieser "Schutz" kann zu Debugsessions fhren, wenn AVRStudio 
   verwendet wird und dort eine andere, nicht zur Hardware passende 
   Taktrate eingestellt ist: Dann wird die folgende Definition 
   nicht verwendet, sondern stattdessen der Defaultwert (8 MHz?) 
   von AVRStudio - daher Ausgabe einer Warnung falls F_CPU
   noch nicht definiert: */
#warning "F_CPU war noch nicht definiert, wird nun nachgeholt mit 4000000"
#define F_CPU 8000000UL  // Systemtakt in Hz - Definition als unsigned long beachten 
                         // Ohne ergeben sich unten Fehler in der Berechnung
#endif
 
#define BAUD 9600UL      // Baudrate
 
// Berechnungen
#define UBRR_VAL ((F_CPU+BAUD*8)/(BAUD*16)-1)   // clever runden
#define BAUD_REAL (F_CPU/(16*(UBRR_VAL+1)))     // Reale Baudrate
#define BAUD_ERROR ((BAUD_REAL*1000)/BAUD) // Fehler in Promille, 1000 = kein Fehler.
 
#if ((BAUD_ERROR<990) || (BAUD_ERROR>1010))
  #error Systematischer Fehler der Baudrate grsser 1% und damit zu hoch! 
#endif 

// FUNKTIONEN
void uart_init(void);
int uart_putc(unsigned char c);
void uart_puts (char *s);
unsigned char uart_getc(void);


// Initialisiert die USARt Schnittstelle
void uart_init(void)
{

  /* Set baud rate */
	UBRRH = (unsigned char)(UBRR_VAL>>8);
	UBRRL = (unsigned char)UBRR_VAL;

	/* Enable receiver and transmitter + RXD-Interrupt */
	UCSRB = (1<<RXEN)|(1<<TXEN)|(1<<RXCIE);

	/* Set frame format: 8data, 2stop bit */
	UCSRC = (1<<URSEL)|(1<<USBS)|(3<<UCSZ0);

}


// schreibt ein Zeichen
int uart_putc(unsigned char c)
{
    while ( !( UCSRA & (1<<UDRE)) ); /* warten bis Senden mglich */                            
 
    UDR = c;                      /* sende Zeichen */
    return 0;
}
 
 
// Schreibt einen String
// puts ist unabhaengig vom Controllertyp
void uart_puts (char *s)
{
    while (*s)
    {   /* so lange *s != '\0' also ungleich dem "String-Endezeichen(Terminator)" */
        uart_putc(*s);
        s++;
    }
}

// Empfngt ein Zeichen
unsigned char uart_getc(void)
{
    while (!(UCSRA & (1<<RXC)));   // warten bis Zeichen verfuegbar

    return UDR;                   // Zeichen aus UDR an Aufrufer zurueckgeben
}
